@@ -94,6 +94,10 @@ class ProgressDialog(MProgressDialog, Window):
         if 'message' in kw:
             self._message_text = kw.pop('message')
 
+        # initialize the start time in case some tries updating
+        # before open() is called
+        self._start_time = 0
+
         super(ProgressDialog, self).__init__( *args, **kw)
 
     def open(self):
@@ -139,6 +143,14 @@ class ProgressDialog(MProgressDialog, Window):
 
         self.progress_bar.update(value)
 
+        # a bit hackish, but on Windows if another window sets focus, the 
+        # other window will come to the top, obscuring the progress dialog. 
+        # Only do this if the control is a top level window, so windows which
+        # embed a progress dialog won't keep popping to the top
+        
+        if self.control.IsTopLevel():
+            self.control.Raise()
+        
         if self.max > 0:
             percent = (float(value) - self.min)/(self.max - self.min)
 
