@@ -26,17 +26,14 @@ import wx
 
 from enthought.traits.api \
     import Int, List, Instance, Str, Any, Button, Tuple, \
-           HasPrivateTraits, Bool, Event, Property, on_trait_change
+           HasPrivateTraits, Bool, Event, Property
 
 from enthought.traits.ui.api \
     import View, Item, UI, InstanceEditor, EnumEditor, Handler, SetEditor, \
            ListUndoItem
     
-# FIXME: ToolkitEditorFactory is a proxy class defined here just for backward
-# compatibility. The class has been moved to the 
-# enthought.traits.ui.editors.table_editor file.
 from enthought.traits.ui.editors.table_editor \
-    import ToolkitEditorFactory
+    import BaseTableEditor, customize_filter
 
 from enthought.traits.ui.menu \
     import Action, ToolBar
@@ -68,17 +65,12 @@ from editor \
 from table_model \
     import TableModel, TraitGridSelection
 
-from helper \
-    import open_fbi, TraitsUIPanel
+from helper import TraitsUIPanel
 
 #-------------------------------------------------------------------------------
 #  Constants:
 #-------------------------------------------------------------------------------
 
-# The filter used to indicate that the user wants to customize the current
-# filter
-customize_filter = TableFilter( name = 'Customize...' )
-
 # Mapping from TableEditor selection modes to Grid selection modes:
 GridModes = {
     'row':     'rows',
@@ -93,7 +85,7 @@ GridModes = {
 #  'TableEditor' class:
 #-------------------------------------------------------------------------------
 
-class TableEditor ( Editor ):
+class TableEditor ( Editor, BaseTableEditor ):
     """ Editor that presents data in a table. Optionally, tables can have
         a set of filters that reduce the set of data displayed, according to 
         their criteria.
@@ -189,10 +181,7 @@ class TableEditor ( Editor ):
         
         self.model = model = TableModel( 
                                 editor         = self,
-                                reverse        = factory.reverse,
-                                cell_theme     = factory.cell_theme,
-                                alt_theme      = factory.alt_theme,
-                                selected_theme = factory.selected_theme )
+                                reverse        = factory.reverse )
         model.on_trait_change( self._model_sorted, 'sorted', dispatch = 'ui' )
         mode     = factory.selection_mode
         row_mode = mode in ( 'row', 'rows' )
@@ -207,8 +196,6 @@ class TableEditor ( Editor ):
 
             # Create the table (i.e. grid) control:
             hsizer = wx.BoxSizer( wx.HORIZONTAL )
-            hsizer.Add( wx.StaticLine( panel, -1, style = wx.LI_VERTICAL ), 0,
-                        wx.EXPAND )
             self._create_grid( panel, hsizer )
             sizer.Add( hsizer, 1, wx.EXPAND )
         else:
@@ -231,8 +218,6 @@ class TableEditor ( Editor ):
 
             # Create the table (i.e. grid) control:
             hsizer = wx.BoxSizer( wx.HORIZONTAL )
-            hsizer.Add( wx.StaticLine( panel, -1, style = wx.LI_VERTICAL ), 0,
-                        wx.EXPAND )
             self._create_grid( panel, hsizer )
             sizer.Add( hsizer, 1, wx.EXPAND )
 
@@ -280,7 +265,7 @@ class TableEditor ( Editor ):
         sv( factory.click,  'click',  'to' )
         sv( factory.dclick, 'dclick', 'to' )
         sv( factory.filter_name,  'filter',  'from' )
-        sv( factory.columns_name, 'columns', 'from', is_list = True )
+        sv( factory.columns_name, 'columns', is_list = True )
         sv( factory.filtered_indices, 'filtered_indices', 'to' )
         sv( factory.selected, 'selected_%s' % mode, is_list = is_list )
         if is_list:
@@ -300,8 +285,15 @@ class TableEditor ( Editor ):
         else:
             self.set_selection()
 
+
+        # Set the min height of the grid panel to 0, this will provide
+        # a scrollbar if the window is resized such that only the first row
+        # is visible
+        panel.SetMinSize((-1, 0))
+        
         # Finish the panel layout setup:
         panel.SetSizer( sizer )
+        
 
     #---------------------------------------------------------------------------
     #  Creates the associated grid control used to implement the table:
@@ -319,7 +311,7 @@ class TableEditor ( Editor ):
             model                        = self.model,
             enable_lines                 = factory.show_lines,
             grid_line_color              = factory.line_color,
-            show_row_headers             = False,
+            show_row_headers             = factory.show_row_labels,
             show_column_headers          = factory.show_column_labels,
             default_cell_font            = factory.cell_font,
             default_cell_text_color      = factory.cell_color,
@@ -355,10 +347,18 @@ class TableEditor ( Editor ):
         # Calculate a reasonable default size for the table:
         if len( self.model.get_filtered_items() ) > 0:
             height = _grid.GetRowSize( 0 )
-        _grid.SetMinSize( wx.Size(
-            max( 150, 80 * len( self.columns ) ), 
-            _grid.GetColLabelSize() + ((factory.rows or 15) * height) ) )
-
+            
+        max_rows = factory.rows or 15
+            
+        min_width = max( 150, 80 * len( self.columns ) )
+        
+        if factory.show_column_labels:
+            min_height = _grid.GetColLabelSize() + (max_rows * height)
+        else:
+            min_height = (max_rows * height)
+            
+        _grid.SetMinSize(wx.Size(min_width, min_height))
+        
         # On Linux, there is what appears to be a bug in wx in which the
         # vertical scrollbar will not be sized properly if the TableEditor is 
         # sized to be shorter than the minimum height specified above. Since
@@ -407,12 +407,11 @@ class TableEditor ( Editor ):
 
             if toolbar.control is not None:
                 self.toolbar = toolbar
+                # add padding so the toolbar is right aligned
                 tb_sizer.Add( ( 1, 1 ), 1, wx.EXPAND )
                 tb_sizer.Add( toolbar.control, 0 )
 
             sizer.Add( tb_sizer, 0, wx.ALIGN_RIGHT | wx.EXPAND )
-            sizer.Add( wx.StaticLine( parent, -1, style = wx.LI_HORIZONTAL ), 0,
-                       wx.EXPAND | wx.BOTTOM, 5 )
 
     #---------------------------------------------------------------------------
     #  Disposes of the contents of an editor:
@@ -1065,42 +1064,6 @@ class TableEditor ( Editor ):
             self.model.filter = new_filter
             self.filter_modified()
         
-    #---------------------------------------------------------------------------
-    #  Event handlers:
-    #---------------------------------------------------------------------------
-
-    #---------------------------------------------------------------------------
-    #  Handles the set of filters associated with the editor being changed:
-    #---------------------------------------------------------------------------
-    
-    ### FIXME: This method was part of the EditorFactory originally (which is
-    # now part of Traits UI). The method was shifted here solely to avoid
-    # declaring the 'customize_filter' constant in the factory file, 
-    # which would importing the TableFilter in the factory file (importing
-    # the TableFilter in the factory file would require changing traits-ui 
-    # imports in TableFilter and in pyface to their source rather than the 
-    # ui.api). As such, this method is toolkit-independent and should be part
-    # of the factory.
-    @on_trait_change('factory.filters[]')
-    def _filters_changed_for_factory ( self, object, name, old, new ):
-        """ Handles the set of filters associated with the editor's factory 
-        being changed.
-        """
-        
-        if self.factory is not None:
-            values = { None: '000:No filter' }
-            i      = 0
-            for filter in self.factory.filters:
-                if not filter.template:
-                    i += 1
-                    values[ filter ] = '%03d:%s' % ( i, filter.name )
-            values[ customize_filter ] = '%03d:%s' % ( (i + 1),
-                                                       customize_filter.name )
-            if self.factory._filter_editor is None:
-                self.factory._filter_editor = EnumEditor( values = values )
-            else:
-                self.factory._filter_editor.values = values
-
     #---------------------------------------------------------------------------
     #  Refresh the list of available filters:
     #---------------------------------------------------------------------------
@@ -1132,9 +1095,7 @@ class TableEditor ( Editor ):
             title   = 'Customize Filters',
             kind    = 'livemodal',
             height  = .25,
-            buttons = [ 'OK', 'Cancel' ],
-            #help_id = "enlib|HID_Customize_Filters_Dlg",
-        ) )
+            buttons = [ 'OK', 'Cancel' ] ) )
         
         if ui.result:
             self._refresh_filters( enum_editor.values )
@@ -1233,6 +1194,9 @@ class TableEditor ( Editor ):
         indices = self.selected_indices[:]
         values  = self.selected_values[:]
         indices.reverse()
+
+        # Make sure that we don't delete any rows while an editor is open in it
+        self.grid.stop_editing_indices(indices)
             
         # Delete the selected rows:
         for i in indices:
@@ -1291,7 +1255,6 @@ class TableEditor ( Editor ):
                          height    = 0.3,
                          resizable = True,
                          buttons   = [ 'Undo', 'OK', 'Cancel' ],
-                         #help_id  = "enlib|HID_Select_and_Order_Columns_Dlg",
                          kind      = 'livemodal' ) )
 
     #---------------------------------------------------------------------------
@@ -1306,108 +1269,7 @@ class TableEditor ( Editor ):
         if object not in selection:
             self.set_selection( object )
             selection = [ object ]
-        self._menu_context = { 'selection': selection,
-                               'object':    object,
-                               'column':    column,
-                               'editor':    self,
-                               'info':      self.ui.info,
-                               'handler':   self.ui.handler }
-
-#----- pyface.action 'controller' interface implementation: --------------------
-
-    #---------------------------------------------------------------------------
-    #  Adds a menu item to the menu being constructed:
-    #---------------------------------------------------------------------------
-
-    def add_to_menu ( self, menu_item ):
-        """ Adds a menu item to the menu bar being constructed.
-        """
-        action = menu_item.item.action
-        self.eval_when( action.enabled_when, menu_item, 'enabled' )
-        self.eval_when( action.checked_when, menu_item, 'checked' )
-
-    #---------------------------------------------------------------------------
-    #  Adds a tool bar item to the tool bar being constructed:
-    #---------------------------------------------------------------------------
-
-    def add_to_toolbar ( self, toolbar_item ):
-        """ Adds a toolbar item to the toolbar being constructed.
-        """
-        self.add_to_menu( toolbar_item )
-
-    #---------------------------------------------------------------------------
-    #  Returns whether the menu action should be defined in the user interface:
-    #---------------------------------------------------------------------------
-
-    def can_add_to_menu ( self, action ):
-        """ Returns whether the action should be defined in the user interface.
-        """
-        if action.defined_when != '':
-            try:
-                if not eval( action.defined_when, globals(), 
-                             self._menu_context ):
-                    return False
-            except:
-                open_fbi()
-
-        if action.visible_when != '':
-            try:
-                if not eval( action.visible_when, globals(), 
-                             self._menu_context ):
-                    return False
-            except:
-                open_fbi()
-
-        return True
-
-    #---------------------------------------------------------------------------
-    #  Returns whether the toolbar action should be defined in the user
-    #  interface:
-    #---------------------------------------------------------------------------
-
-    def can_add_to_toolbar ( self, action ):
-        """ Returns whether the toolbar action should be defined in the user
-            interface.
-        """
-        return self.can_add_to_menu( action )
-
-    #---------------------------------------------------------------------------
-    #  Performs the action described by a specified Action object:
-    #---------------------------------------------------------------------------
-
-    def perform ( self, action, action_event = None ):
-        """ Performs the action described by a specified Action object.
-        """
-        self.ui.do_undoable( self._perform, action )
-
-    def _perform ( self, action ):
-        method_name        = action.action
-        info               = self.ui.info
-        handler            = self.ui.handler
-        context            = self._menu_context
-        self._menu_context = None
-        selection          = context[ 'selection' ]
-
-        if method_name.find( '.' ) >= 0:
-            if method_name.find( '(' ) < 0:
-                method_name += '()'
-            try:
-                eval( method_name, globals(), context )
-            except:
-                # fixme: Should the exception be logged somewhere?
-                pass
-            return
-
-        method = getattr( handler, method_name, None )
-        if method is not None:
-            method( info, selection )
-            return
-
-        if action.on_perform is not None:
-            action.on_perform( selection )
-            return
-
-        action.perform( selection )
+        self.set_menu_context( selection, object, column )
         
     #---------------------------------------------------------------------------
     #  Set one or more attributes without notifying the grid model:
@@ -1423,26 +1285,6 @@ class TableEditor ( Editor ):
             
         self._no_notify = False
 
-#-- Menu support methods: ------------------------------------------------------
-
-    #---------------------------------------------------------------------------
-    #  Evaluates a condition within a defined context and sets a specified
-    #  object trait based on the (assumed) boolean result:
-    #---------------------------------------------------------------------------
-
-    def eval_when ( self, condition, object, trait ):
-        """ Evaluates a condition within a defined context and sets a specified
-        object trait based on the result, which is iassumed to be a Boolean.
-        """
-        if condition != '':
-            value = True
-            try:
-                if not eval( condition, globals(), self._menu_context ):
-                    value = False
-            except:
-                open_fbi()
-            setattr( object, trait, value )
-
 #-- Private Methods: -----------------------------------------------------------
 
     #---------------------------------------------------------------------------
@@ -1488,10 +1330,6 @@ class TableFilterEditor ( Handler ):
     #  'Handler' interface:
     #---------------------------------------------------------------------------
 
-    #---------------------------------------------------------------------------
-    #  Initializes the controls of a user interface:
-    #---------------------------------------------------------------------------
-
     def init ( self, info ):
         """ Initializes the controls of a user interface.
         """
@@ -1501,10 +1339,6 @@ class TableFilterEditor ( Handler ):
         self._filter = self.filter
         self._filter_copy = self.filter.clone_traits()
 
-    #---------------------------------------------------------------------------
-    #  Handles a dialog-based user interface being closed by the user:
-    #---------------------------------------------------------------------------
-
     def closed ( self, info, is_ok ):
         """ Handles a dialog-based user interface being closed by the user.
         """
@@ -1516,10 +1350,6 @@ class TableFilterEditor ( Handler ):
     #  Event handlers:
     #---------------------------------------------------------------------------
 
-    #---------------------------------------------------------------------------
-    #  Handles a new filter being selected:
-    #---------------------------------------------------------------------------
-
     def object_filter_changed ( self, info ):
         """ Handles a new filter being selected.
         """
@@ -1528,22 +1358,21 @@ class TableFilterEditor ( Handler ):
         info.delete.enabled = ((not filter.template) and
                                (len( info.filter.factory.values ) > 1))
 
-    #---------------------------------------------------------------------------
-    #  Handles the user clicking the 'Edit' button:
-    #---------------------------------------------------------------------------
-
     def object_edit_changed ( self, info ):
         """ Handles the user clicking the **Edit** button.
         """
         if info.initialized:
-            ui = self.filter.edit( self.editor.model.get_filtered_item( 0 ) )
+            items = self.editor.model.get_filtered_items()
+            if len(items) > 0:
+                item = items[0]
+            else:
+                item = None
+            # `item` is now either the first item in the table, or None if
+            # the table is empty.
+            ui = self.filter.edit(item)
             if ui.result:
                 self._refresh_filters( info )
 
-    #---------------------------------------------------------------------------
-    #  Handles the user clicking the 'New' button:
-    #---------------------------------------------------------------------------
-
     def object_new_changed ( self, info ):
         """ Handles the user clicking the **New** button.
         """
@@ -1570,10 +1399,6 @@ class TableFilterEditor ( Handler ):
             self.filter = new_filter
             do_later( self._delayed_edit, info )
 
-    #---------------------------------------------------------------------------
-    #  Handles the user clicking the 'Apply' button:
-    #---------------------------------------------------------------------------
-
     def object_apply_changed ( self, info ):
         """ Handles the user clicking the **Apply** button.
         """
@@ -1582,10 +1407,6 @@ class TableFilterEditor ( Handler ):
             self.editor._refresh_filters( info.filter.factory.values )
             self.editor.filter = self.filter
 
-    #---------------------------------------------------------------------------
-    #  Handles the user clicking the 'Delete' button:
-    #---------------------------------------------------------------------------
-
     def object_delete_changed ( self, info ):
         """ Handles the user clicking the **Delete** button.
         """
@@ -1607,10 +1428,6 @@ class TableFilterEditor ( Handler ):
     #  Private methods:
     #---------------------------------------------------------------------------
 
-    #---------------------------------------------------------------------------
-    #  Refresh the filter editor's list of filters:
-    #---------------------------------------------------------------------------
-
     def _refresh_filters ( self, info ):
         """ Refresh the filter editor's list of filters.
         """
@@ -1618,12 +1435,9 @@ class TableFilterEditor ( Handler ):
         values, factory.values = factory.values, []
         factory.values = values
 
-    #---------------------------------------------------------------------------
-    #  Edits the current filter, and deletes it if the user cancels the edit:
-    #---------------------------------------------------------------------------
-
     def _delayed_edit ( self, info ):
-        """ Edits the current filter, and deletes it if the user cancels the edit.
+        """ Edits the current filter, and deletes it if the user cancels the 
+            edit.
         """
         ui = self.filter.edit( self.editor.model.get_filtered_item( 0 ) )
         if not ui.result:
@@ -1632,7 +1446,7 @@ class TableFilterEditor ( Handler ):
             self._refresh_filters( info )
 
         # Allow deletion as long as there is more than 1 filter:
-        if len( info.filter.factory.values ) > 1:
+        if (not self.filter.template) and len( info.filter.factory.values ) > 1:
             info.delete.enabled = True
 
 #-------------------------------------------------------------------------------
@@ -1753,48 +1567,27 @@ class TableEditorToolbar ( HasPrivateTraits ):
     #  PyFace/Traits menu/toolbar controller interface:
     #---------------------------------------------------------------------------
 
-    #---------------------------------------------------------------------------
-    #  Adds a menu item to the menu bar being constructed:
-    #---------------------------------------------------------------------------
-
     def add_to_menu ( self, menu_item ):
         """ Adds a menu item to the menu bar being constructed.
         """
         pass
 
-    #---------------------------------------------------------------------------
-    #  Adds a tool bar item to the tool bar being constructed:
-    #---------------------------------------------------------------------------
-
     def add_to_toolbar ( self, toolbar_item ):
         """ Adds a toolbar item to the too bar being constructed.
         """
         pass
 
-    #---------------------------------------------------------------------------
-    #  Returns whether the menu action should be defined in the user interface:
-    #---------------------------------------------------------------------------
-
     def can_add_to_menu ( self, action ):
         """ Returns whether the action should be defined in the user interface.
         """
         return True
 
-    #---------------------------------------------------------------------------
-    #  Returns whether the toolbar action should be defined in the user
-    #  interface:
-    #---------------------------------------------------------------------------
-
     def can_add_to_toolbar ( self, action ):
         """ Returns whether the toolbar action should be defined in the user
             interface.
         """
         return True
 
-    #---------------------------------------------------------------------------
-    #  Performs the action described by a specified Action object:
-    #---------------------------------------------------------------------------
-
     def perform ( self, action, action_event = None ):
         """ Performs the action described by a specified Action object.
         """
@@ -1904,6 +1697,3 @@ SimpleEditor = TableEditor
 
 # Define the ReadonlyEditor class.
 ReadonlyEditor = TableEditor
-
-### EOF #######################################################################
-
